@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -44,6 +36,9 @@
 
 #include <QTemporaryDir>
 #include <QFileSystemWatcher>
+#include <QElapsedTimer>
+#include <QTextStream>
+#include <QDir>
 
 /* All tests need to run in temporary directories not used
  * by the application to avoid non-deterministic failures on Windows
@@ -67,6 +62,7 @@ private slots:
     void removePath();
     void addPaths();
     void removePaths();
+    void removePathsFilesInSameDirectory();
 
     void watchFileAndItsDirectory_data() { basicTest_data(); }
     void watchFileAndItsDirectory();
@@ -95,6 +91,10 @@ tst_QFileSystemWatcher::tst_QFileSystemWatcher()
         m_tempDirPattern += QLatin1Char('/');
     m_tempDirPattern += QStringLiteral("tst_qfilesystemwatcherXXXXXX");
 #endif // QT_NO_FILESYSTEMWATCHER
+
+#if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_NO_SDK)
+    QDir::setCurrent(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
+#endif
 }
 
 #ifndef QT_NO_FILESYSTEMWATCHER
@@ -112,9 +112,11 @@ void tst_QFileSystemWatcher::basicTest_data()
         + QChar(ushort(0x00DC)) // LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS
         + QStringLiteral(".txt");
 
+#if !defined(Q_OS_QNX) || !defined(QT_NO_INOTIFY)
     QTest::newRow("native backend-testfile") << "native" << testFile;
-    QTest::newRow("poller backend-testfile") << "poller" << testFile;
     QTest::newRow("native backend-specialchars") << "native" << specialCharacterFile;
+#endif
+    QTest::newRow("poller backend-testfile") << "poller" << testFile;
 }
 
 void tst_QFileSystemWatcher::basicTest()
@@ -138,7 +140,7 @@ void tst_QFileSystemWatcher::basicTest()
     watcher.setObjectName(QLatin1String("_qt_autotest_force_engine_") + backend);
     QVERIFY(watcher.addPath(testFile.fileName()));
 
-    QSignalSpy changedSpy(&watcher, SIGNAL(fileChanged(QString)));
+    QSignalSpy changedSpy(&watcher, &QFileSystemWatcher::fileChanged);
     QVERIFY(changedSpy.isValid());
     QEventLoop eventLoop;
     QTimer timer;
@@ -276,7 +278,7 @@ void tst_QFileSystemWatcher::watchDirectory()
     watcher.setObjectName(QLatin1String("_qt_autotest_force_engine_") + backend);
     QVERIFY(watcher.addPath(testDir.absolutePath()));
 
-    QSignalSpy changedSpy(&watcher, SIGNAL(directoryChanged(QString)));
+    QSignalSpy changedSpy(&watcher, &QFileSystemWatcher::directoryChanged);
     QVERIFY(changedSpy.isValid());
     QEventLoop eventLoop;
     QTimer timer;
@@ -389,6 +391,60 @@ void tst_QFileSystemWatcher::addPaths()
     QCOMPARE(watcher.addPaths(paths), QStringList());
 }
 
+// A signal spy that records the paths and times received for better diagnostics.
+class FileSystemWatcherSpy : public QObject {
+    Q_OBJECT
+public:
+    enum Mode {
+        SpyOnDirectoryChanged,
+        SpyOnFileChanged
+    };
+
+    explicit FileSystemWatcherSpy(QFileSystemWatcher *watcher, Mode mode)
+    {
+        connect(watcher, mode == SpyOnDirectoryChanged ?
+                &QFileSystemWatcher::directoryChanged : &QFileSystemWatcher::fileChanged,
+                this, &FileSystemWatcherSpy::spySlot);
+        m_elapsedTimer.start();
+    }
+
+    int count() const { return m_entries.size(); }
+    void clear()
+    {
+        m_entries.clear();
+        m_elapsedTimer.restart();
+    }
+
+    QByteArray receivedFilesMessage() const
+    {
+        QString result;
+        QTextStream str(&result);
+        str << "At " << m_elapsedTimer.elapsed() << "ms, received "
+            << count() << " changes: ";
+        for (int i =0, e = m_entries.size(); i < e; ++i) {
+            if (i)
+                str << ", ";
+            str << m_entries.at(i).timeStamp << "ms: " << QDir::toNativeSeparators(m_entries.at(i).path);
+        }
+        return result.toLocal8Bit();
+    }
+
+private slots:
+    void spySlot(const QString &p) { m_entries.append(Entry(m_elapsedTimer.elapsed(), p)); }
+
+private:
+    struct Entry {
+        Entry() : timeStamp(0) {}
+        Entry(qint64 t, const QString &p) : timeStamp(t), path(p) {}
+
+        qint64 timeStamp;
+        QString path;
+    };
+
+    QElapsedTimer m_elapsedTimer;
+    QList<Entry> m_entries;
+};
+
 void tst_QFileSystemWatcher::removePaths()
 {
     QFileSystemWatcher watcher;
@@ -405,6 +461,38 @@ void tst_QFileSystemWatcher::removePaths()
     watcher.removePaths(paths);
 }
 
+void tst_QFileSystemWatcher::removePathsFilesInSameDirectory()
+{
+    // QTBUG-46449/Windows: Check the return values of removePaths().
+    // When adding the 1st file, a thread is started to watch the temp path.
+    // After adding and removing the 2nd file, the thread is still running and
+    // success should be reported.
+    QTemporaryFile file1(m_tempDirPattern);
+    QTemporaryFile file2(m_tempDirPattern);
+    QVERIFY2(file1.open(), qPrintable(file1.errorString()));
+    QVERIFY2(file2.open(), qPrintable(file1.errorString()));
+    const QString path1 = file1.fileName();
+    const QString path2 = file2.fileName();
+    file1.close();
+    file2.close();
+    QFileSystemWatcher watcher;
+    QVERIFY(watcher.addPath(path1));
+    QCOMPARE(watcher.files().size(), 1);
+    QVERIFY(watcher.addPath(path2));
+    QCOMPARE(watcher.files().size(), 2);
+    QVERIFY(watcher.removePath(path1));
+    QCOMPARE(watcher.files().size(), 1);
+    QVERIFY(watcher.removePath(path2));
+    QCOMPARE(watcher.files().size(), 0);
+}
+
+static QByteArray msgFileOperationFailed(const char *what, const QFile &f)
+{
+    return what + QByteArrayLiteral(" failed on \"")
+        + QDir::toNativeSeparators(f.fileName()).toLocal8Bit()
+        + QByteArrayLiteral("\": ") + f.errorString().toLocal8Bit();
+}
+
 void tst_QFileSystemWatcher::watchFileAndItsDirectory()
 {
     QFETCH(QString, backend);
@@ -420,14 +508,10 @@ void tst_QFileSystemWatcher::watchFileAndItsDirectory()
 
     QString testFileName = testDir.filePath("testFile.txt");
     QString secondFileName = testDir.filePath("testFile2.txt");
-    QFile::remove(secondFileName);
 
     QFile testFile(testFileName);
-    testFile.setPermissions(QFile::ReadOwner | QFile::WriteOwner);
-    testFile.remove();
-
-    QVERIFY(testFile.open(QIODevice::WriteOnly | QIODevice::Truncate));
-    testFile.write(QByteArray("hello"));
+    QVERIFY2(testFile.open(QIODevice::WriteOnly | QIODevice::Truncate), msgFileOperationFailed("open", testFile));
+    QVERIFY2(testFile.write(QByteArrayLiteral("hello")) > 0, msgFileOperationFailed("write", testFile));
     testFile.close();
 
     QFileSystemWatcher watcher;
@@ -436,10 +520,9 @@ void tst_QFileSystemWatcher::watchFileAndItsDirectory()
     QVERIFY(watcher.addPath(testDir.absolutePath()));
     QVERIFY(watcher.addPath(testFileName));
 
-    QSignalSpy fileChangedSpy(&watcher, SIGNAL(fileChanged(QString)));
-    QSignalSpy dirChangedSpy(&watcher, SIGNAL(directoryChanged(QString)));
+    QSignalSpy fileChangedSpy(&watcher, &QFileSystemWatcher::fileChanged);
+    FileSystemWatcherSpy dirChangedSpy(&watcher, FileSystemWatcherSpy::SpyOnDirectoryChanged);
     QVERIFY(fileChangedSpy.isValid());
-    QVERIFY(dirChangedSpy.isValid());
     QEventLoop eventLoop;
     QTimer timer;
     connect(&timer, SIGNAL(timeout()), &eventLoop, SLOT(quit()));
@@ -449,8 +532,8 @@ void tst_QFileSystemWatcher::watchFileAndItsDirectory()
     // wait before modifying the directory...
     QTest::qWait(2000);
 
-    QVERIFY(testFile.open(QIODevice::WriteOnly | QIODevice::Truncate));
-    testFile.write(QByteArray("hello again"));
+    QVERIFY2(testFile.open(QIODevice::WriteOnly | QIODevice::Truncate), msgFileOperationFailed("open", testFile));
+    QVERIFY2(testFile.write(QByteArrayLiteral("hello again")), msgFileOperationFailed("write", testFile));
     testFile.close();
 
 #ifdef Q_OS_MAC
@@ -459,21 +542,12 @@ void tst_QFileSystemWatcher::watchFileAndItsDirectory()
 #endif
 
     QTRY_VERIFY(fileChangedSpy.count() > 0);
-
-    //according to Qt 4 documentation:
-    //void QFileSystemWatcher::directoryChanged ( const QString & path )   [signal]
-    //This signal is emitted when the directory at a specified path, is modified
-    //(e.g., when a file is added, -->modified<-- or deleted) or removed from disk.
-    //Note that if there are several changes during a short period of time, some
-    //of the changes might not emit this signal. However, the last change in the
-    //sequence of changes will always generate this signal.
-    QVERIFY(dirChangedSpy.count() < 2);
+    QVERIFY2(dirChangedSpy.count() == 0, dirChangedSpy.receivedFilesMessage());
 
     fileChangedSpy.clear();
-    dirChangedSpy.clear();
     QFile secondFile(secondFileName);
-    secondFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
-    secondFile.write("Foo");
+    QVERIFY2(secondFile.open(QIODevice::WriteOnly | QIODevice::Truncate), msgFileOperationFailed("open", secondFile));
+    QVERIFY2(secondFile.write(QByteArrayLiteral("Foo")) > 0, msgFileOperationFailed("write", secondFile));
     secondFile.close();
 
     timer.start(3000);
@@ -491,17 +565,17 @@ void tst_QFileSystemWatcher::watchFileAndItsDirectory()
 
     dirChangedSpy.clear();
 
-    QFile::remove(testFileName);
+    QVERIFY(QFile::remove(testFileName));
 
     QTRY_VERIFY(fileChangedSpy.count() > 0);
-    QCOMPARE(dirChangedSpy.count(), 1);
+    QTRY_COMPARE(dirChangedSpy.count(), 1);
 
     fileChangedSpy.clear();
     dirChangedSpy.clear();
 
     // removing a deleted file should fail
     QVERIFY(!watcher.removePath(testFileName));
-    QFile::remove(secondFileName);
+    QVERIFY(QFile::remove(secondFileName));
 
     timer.start(3000);
     eventLoop.exec();
@@ -596,7 +670,7 @@ void tst_QFileSystemWatcher::QTBUG2331()
     QVERIFY(watcher.addPath(temporaryDirectory.path()));
 
     // watch signal
-    QSignalSpy changedSpy(&watcher, SIGNAL(directoryChanged(QString)));
+    QSignalSpy changedSpy(&watcher, &QFileSystemWatcher::directoryChanged);
     QVERIFY(changedSpy.isValid());
 
     // remove directory, we should get one change signal, and we should no longer
@@ -648,13 +722,14 @@ private:
 // emitted with the destination path instead of the starting path
 void tst_QFileSystemWatcher::signalsEmittedAfterFileMoved()
 {
+    const int fileCount = 10;
     QTemporaryDir temporaryDirectory(m_tempDirPattern);
     QVERIFY(temporaryDirectory.isValid());
     QDir testDir(temporaryDirectory.path());
     QVERIFY(testDir.mkdir("movehere"));
     QString movePath = testDir.filePath("movehere");
 
-    for (int i = 0; i < 10; i++) {
+    for (int i = 0; i < fileCount; ++i) {
         QFile f(testDir.filePath(QString("test%1.txt").arg(i)));
         QVERIFY(f.open(QIODevice::WriteOnly));
         f.write(QByteArray("i am ") + QByteArray::number(i));
@@ -667,6 +742,7 @@ void tst_QFileSystemWatcher::signalsEmittedAfterFileMoved()
 
     // add files to watcher
     QFileInfoList files = testDir.entryInfoList(QDir::Files | QDir::NoSymLinks);
+    QCOMPARE(files.size(), fileCount);
     foreach (const QFileInfo &finfo, files)
         QVERIFY(watcher.addPath(finfo.absoluteFilePath()));
 
@@ -675,14 +751,16 @@ void tst_QFileSystemWatcher::signalsEmittedAfterFileMoved()
     connect(&watcher, SIGNAL(fileChanged(QString)), &signalReceiver, SLOT(fileChanged(QString)));
 
     // watch signals
-    QSignalSpy changedSpy(&watcher, SIGNAL(fileChanged(QString)));
-    QVERIFY(changedSpy.isValid());
+    FileSystemWatcherSpy changedSpy(&watcher, FileSystemWatcherSpy::SpyOnFileChanged);
+    QCOMPARE(changedSpy.count(), 0);
 
     // move files to second directory
     foreach (const QFileInfo &finfo, files)
         QVERIFY(testDir.rename(finfo.fileName(), QString("movehere/%2").arg(finfo.fileName())));
 
-    QTRY_COMPARE(changedSpy.count(), 10);
+    QCoreApplication::processEvents();
+    QVERIFY2(changedSpy.count() <= fileCount, changedSpy.receivedFilesMessage());
+    QTRY_COMPARE(changedSpy.count(), fileCount);
 }
 #endif // QT_NO_FILESYSTEMWATCHER
 
