@@ -1,40 +1,32 @@
 /****************************************************************************
 **
 ** Copyright (C) 2014 BlackBerry Limited. All rights reserved.
-** Contact: http://www.qt-project.org/legal
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -85,6 +77,7 @@ private:
     QNetworkAccessManager m_manager;
     int m_multipleRequestsCount;
     int m_multipleRepliesFinishedCount;
+    const QString m_rfc3252FilePath;
 
 protected Q_SLOTS:
     void proxyAuthenticationRequired(const QNetworkProxy &, QAuthenticator *authenticator);
@@ -92,6 +85,7 @@ protected Q_SLOTS:
 };
 
 tst_Spdy::tst_Spdy()
+    : m_rfc3252FilePath(QFINDTESTDATA("../qnetworkreply/rfc3252.txt"))
 {
 #if defined(QT_BUILD_INTERNAL) && !defined(QT_NO_SSL) && OPENSSL_VERSION_NUMBER >= 0x1000100fL && !defined(OPENSSL_NO_TLSEXT) && !defined(OPENSSL_NO_NEXTPROTONEG)
     qRegisterMetaType<QNetworkReply *>(); // for QSignalSpy
@@ -110,6 +104,7 @@ tst_Spdy::~tst_Spdy()
 
 void tst_Spdy::initTestCase()
 {
+    QVERIFY(!m_rfc3252FilePath.isEmpty());
     QVERIFY(QtNetworkSettings::verifyTestNetworkSettings());
 }
 
@@ -171,8 +166,7 @@ void tst_Spdy::settingsAndNegotiation()
     QFETCH(QByteArray, expectedProtocol);
 
 #ifndef QT_NO_OPENSSL
-    bool expectedSpdyUsed = (expectedProtocol == QSslConfiguration::NextProtocolSpdy3_0)
-            ? true : false;
+    bool expectedSpdyUsed = (expectedProtocol == QSslConfiguration::NextProtocolSpdy3_0);
     QCOMPARE(reply->attribute(QNetworkRequest::SpdyWasUsedAttribute).toBool(), expectedSpdyUsed);
 #endif // QT_NO_OPENSSL
 
@@ -215,7 +209,7 @@ void tst_Spdy::download_data()
 
     QTest::newRow("mediumfile") << QUrl("https://" + QtNetworkSettings::serverName()
                                         + "/qtest/rfc3252.txt")
-                                << QFINDTESTDATA("../qnetworkreply/rfc3252.txt")
+                                << m_rfc3252FilePath
                                 << QNetworkProxy();
 
     QHostInfo hostInfo = QHostInfo::fromName(QtNetworkSettings::serverName());
@@ -223,23 +217,23 @@ void tst_Spdy::download_data()
 
     QTest::newRow("mediumfile-http-proxy") << QUrl("https://" + QtNetworkSettings::serverName()
                                                    + "/qtest/rfc3252.txt")
-                                           << QFINDTESTDATA("../qnetworkreply/rfc3252.txt")
+                                           << m_rfc3252FilePath
                                            << QNetworkProxy(QNetworkProxy::HttpProxy, proxyserver, 3128);
 
     QTest::newRow("mediumfile-http-proxy-auth") << QUrl("https://" + QtNetworkSettings::serverName()
                                                         + "/qtest/rfc3252.txt")
-                                                << QFINDTESTDATA("../qnetworkreply/rfc3252.txt")
+                                                << m_rfc3252FilePath
                                                 << QNetworkProxy(QNetworkProxy::HttpProxy,
                                                                  proxyserver, 3129);
 
     QTest::newRow("mediumfile-socks-proxy") << QUrl("https://" + QtNetworkSettings::serverName()
                                                     + "/qtest/rfc3252.txt")
-                                            << QFINDTESTDATA("../qnetworkreply/rfc3252.txt")
+                                            << m_rfc3252FilePath
                                             << QNetworkProxy(QNetworkProxy::Socks5Proxy, proxyserver, 1080);
 
     QTest::newRow("mediumfile-socks-proxy-auth") << QUrl("https://" + QtNetworkSettings::serverName()
                                                          + "/qtest/rfc3252.txt")
-                                                 << QFINDTESTDATA("../qnetworkreply/rfc3252.txt")
+                                                 << m_rfc3252FilePath
                                                  << QNetworkProxy(QNetworkProxy::Socks5Proxy,
                                                                   proxyserver, 1081);
 
@@ -408,7 +402,7 @@ void tst_Spdy::upload_data()
 
     // 2. test uploading of files
 
-    QFile *file = new QFile(QFINDTESTDATA("../qnetworkreply/rfc3252.txt"));
+    QFile *file = new QFile(m_rfc3252FilePath);
     file->open(QIODevice::ReadOnly);
     QTest::newRow("file-26K") << md5Url << QByteArray() << QByteArray("POST")
                               << static_cast<QObject *>(file)
